/***
 * htmlwais
 *
 *	Sends a wais question to a source of html files and
 *	analyzis the results. The hitfiles are scanned for their
 *	title and for the main headings. These are used to build
 *	an abstract of the file. The list is returned to the user.
 *
 * 	This is a freeby, no guarantees, minimal support but
 *	will fix bugs.
 *
 *	Its place of origin is: 
 *		ftp.swin.edu.au:/pub/cwis/htmlwais-0.9.tar.gz
 *
 *	Ver 0.92 last revised:	16-Jan-95	harry@swin.edu.au
 ***/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <memory.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "conf.h"
#include "htmlwais.h"

#define NO	0
#define OK	1

char *question;

main(argc, argv)
int argc;
char *argv[];
{
	entry entries[MAX_ENTRIES];
	register int x,m=0;
	struct waisresult *head;
	char *t, *key, *tmpfile, cmd[512], *wais_source, *waispath;
	int cl;

	if (argc==2) 
		wais_source=strdup(argv[1]);
	else
		wais_source=strdup(WAISSRC);

	printf("Content-type: text/html\n\n");

	waispath = (char *) malloc(strlen(WAISDIR)+
				strlen(wais_source)+2);  
	sprintf(waispath,"%s/%s",WAISDIR,wais_source);
	if (access(waispath,R_OK)<0) {
		printf("%sERROR: WAIS Source: \"%s\" not found.%s\n",HEAD4,
				wais_source,EHEAD4);
		exit(0);

	}

	t = getenv("REQUEST_METHOD"); 
	if (!t) {
		printf("Environment variable: REQUEST_METHOD not set\n");
		printf("Should be used in a WEB FORM\n");
		exit(-1);
	}
 	
	if(strcmp(t,"POST")) {
        	printf("This program should be referenced with a METHOD of POST.\n");
        	printf("If you don't understand this, see this ");
        	printf("<A HREF=\"http://www.ncsa.uiuc.edu/SDG/Software/Mosaic/Docs/fill-out-forms/overview.html\">forms overview</A>.%c",10);
        	exit(1);
	}
	
	t = getenv("CONTENT_TYPE");
	if (!t) {
		printf("Environment variable: CONTENT_TYPE not set\n");
		printf("Should be used in a WEB FORM\n");
		exit(-1);
	}

	if(strcmp(t, "application/x-www-form-urlencoded")) {
        	printf("This script can only be used to decode form results. \n");
        	exit(1);
	}

	t = getenv("CONTENT_LENGTH");
	if (!t) {
		printf("Environment variable: CONTENT_LENGTH not set\n");
		printf("Should be used in a WEB FORM\n");
		exit(-1);
	}
	cl = atoi(t);

	for(x=0;cl && (!feof(stdin));x++) {
		entries[x].val = fmakeword(stdin,'&',&cl);
		plustospace(entries[x].val);
		unescape_url(entries[x].val);
		entries[x].name = makeword(entries[x].val,'=');
	}
    
	if (!(tmpfile = tempnam(NULL,"hw")))
		exit(-1);

	if (!entries[0].val) 
		exit(0);

/* Send the WAIS question  */

	key = fix(entries[0].val);

	sprintf(cmd,"%s -m %d -c %s -f - -S %s -g \"%s\" > %s",
		WAISQ,MAX_LIST,WAISDIR, wais_source,key,tmpfile);

	if (system(cmd)<0) {
		printf("<P>htmlwais: Error running WAISQ<P>\n") ;
		exit(0);
	}

/* Scan the results from the temp file created by WAISQ */

	head = scan_waisqout(tmpfile);
	unlink(tmpfile);

	printht(HTMLHEADER);

	printf("%s%sYou searched for: %s%s%s (Top %d shown)%s",
		   BREAK,RULE,BOLD,entries[0].val,EBOLD,MAX_LIST,PARA);

/* scan the hit files, pluck out titles and headings and print HTML */

	scan_and_print(head);

	printht(HTMLTRAILER);

}

/***
 * scan_waisquit(char *wf)
 *
 *	wf - Path of WAISQ output file. 
 *
 * Scans the output of WAISQ and plucks out the bits we want.
 * It returns a list of this information.
 *
 ****/
struct waisresult *scan_waisqout(wf)
char *wf;
{
	FILE *f1;
	char line[LEN], *tok, *fbuf;
	struct waisresult *headwq, *wq=NULL;
	int pass1=TRUE;

	if ((f1 = fopen(wf,"r"))==NULL)
		return(NULL);

	while ( (fgets(line,LEN,f1))!=NULL) {

	   tok = strtok(line," \t\n\"");

	   if (!tok) 
		continue;

	   if (strncasecmp(tok,SCORE,strlen(SCORE))==0) {
	      if (!(wq=wqnext(wq)))
			return(NULL); 
	      if (pass1) {
		 headwq=wq;
		 pass1=FALSE;
	      }
	      wq->score = strdup(strtok(NULL," \t\n\""));	

	   } else if (strncasecmp(tok, HEADLINE, strlen(HEADLINE))==0) {
	   	wq->fname = strdup(strtok(NULL," \t\n\""));	
	   	wq->dir = strdup(strtok(NULL," \t\n\""));	
	   } else if (strncasecmp(tok, NUMOFB,strlen(NUMOFB))==0) 
	   	wq->numofb = strdup(strtok(NULL," \t\n\""));	
	   else if (strncasecmp(tok, NUMOFL,strlen(NUMOFL))==0) 
	   	wq->numofl = strdup(strtok(NULL," \t\n\""));	
	}

	fclose(f1);
	return(headwq);
}

/***
 * wqnext(struct waisresult *wq)
 *	
 *	wq - Pointer to waisresult structure
 *
 *	Creates a new node on the linked list.
 ***/
struct waisresult *wqnext(wq)
struct waisresult *wq;
{

	if (!wq) {
	    	if ((wq = (struct waisresult *) 
	 		malloc(sizeof(struct waisresult)))==NULL)
			return(NULL);
	} else {
	   	if ((wq->next = (struct waisresult *) 
			malloc(sizeof(struct waisresult)))==NULL)
			return(NULL);
		wq=wq->next;
	}
	wq->next=NULL;
	wq->fname=NULL;
	wq->numofl=NULL;
	wq->numofb=NULL;
	wq->score=NULL;
	wq->dir=NULL;

	return(wq);
}
/***
 * abnext(struct abstract *ab)
 *	
 *	ab - Pointer to abstract structure
 *
 *	Creates a new node on the linked list.
 ***/
struct abstract *abnext(ab)
struct abstract *ab;
{

	if (!ab) {
	    	if ((ab = (struct abstract *) 
	 		malloc(sizeof(struct abstract)))==NULL)
			return(NULL);
	} else {
	   	if ((ab->next = (struct abstract *) 
			malloc(sizeof(struct abstract)))==NULL)
			return(NULL);
		ab=ab->next;
	    }

	    ab->data = (char *) calloc(MAX_ABS, sizeof(char));
	    ab->title=NULL;
	    ab->url=NULL;
	    ab->next=NULL;

	    return(ab);
}
/***
 * scan_hitfile(char *fname)
 *
 *	fname - File name of a 'hit' document in the WAIS search
 *
 * Scans a HTML hit file and plucks out TITLE and HEADINGS 
 * to build an abstract of the file. It returns the info in a little
 * structure.
 ***/
struct abstract *scan_hitfile(fname)
char *fname;		/* file to scan */
{
	FILE *f1;
	char *s, *fbuf;
	struct abstract *ab=NULL;
	int l, refcount=0, type; 
	struct stat fst;
	unsigned long pos;

/* Use a fast file read to minimise search time */

	if (stat(fname,&fst)!=0)
		return(NULL);

	if ((f1=fopen(fname,"r")) == NULL) {
		printf("Error opening file: %s\n",fname);
		return(NULL);
		
	}	

	if ((fbuf = (char *) malloc(fst.st_size))==NULL) {
		printf("Failed to malloc\n");
		return(NULL);
	}

	fread( (char *) fbuf, fst.st_size, 1,f1);

	fclose(f1);

	ab = abnext(ab);
	ab->url = urlstr(fname);

/* Keep scanning the file till the end. Could possible speed it up
   by limitting file size and number of headings */
 
	pos=0;
	while ((s = suckstr(fbuf, &type, fst.st_size, &pos))) {
		if (type==TTITLE)  
			ab->title=strdup(s);
		else if (type==THEAD) {
			l = MAX_ABS - ((strlen(ab->data)+2)+SPARE);
			if (l > strlen(s)) {
				if (refcount)
					strcat(ab->data,", ");
				strcat(ab->data, strlowu(s));		
			}
			refcount++;	
		}	
	}
	free(fbuf);

	insert_breaks(ab->data);

	return(ab);
}
/***
 * scan_and_print(struct waisresult *wq)
 *
 * 	wq - Pointer to linked list of waisresults structure
 *
 *	Scan each file and print the results.
 *	
 ***/
void scan_and_print(wq)
struct waisresult *wq;
{

	char tmp[LEN];
	struct abstract *ab;
	int i=0;

	if (!wq || (atoi(wq->score)==0))
		printf("%s--- Nothing Found.---%s\n",PARA,PARA);

	while (wq && (i++ <= MAX_LIST)) {
		if (show_it(wq->dir)!=0) {
			sprintf(tmp,"%s%s",wq->dir,wq->fname);	
			ab = scan_hitfile(tmp);	
			print_abs(ab, wq);
			free(ab);
		} else
		   	i--;	/* still show MAX_LIST if we have */
		wq=wq->next;
	}
}

/***
 * insert_breaks(char *s)
 ***/
char *insert_breaks(s)
char *s;
{

	int i=0,j=0;
	char *t=(char *) calloc(MAX_ABS, sizeof(char));

	if (!s || (strlen(s)==0))
		return(NULL);

	while (i < strlen(s)) {
		if (i==0) {
			strcat(t," ");
			j=1;
		}
		if ( (i>0) && ((i%54) == 0)) {
			while ((*(s+i)!=0) && (!isspace(*(s+i))) 
					   && (!ispunct(*(s+i)))) {
				*(t+j++) = *(s+i++);
			}
			*(t+j++) = *(s+i++);
			if (i!=strlen(s)) {
			   strcat(t," "); /* in case of '&' char */
			   strcat(t,BREAK);strcat(t,DEF);strcat(t,"\n");
			   j=j+strlen(DEF)+strlen(BREAK)+2;
			}
		} 
		*(t+j++) = *(s+i++);	
	} 
	strcat(t,BREAK);strcat(t,"\n");

	memcpy(s,t,MAX_ABS);

	return;
}

/*** 
 * print_abs()
 *
 *	Print the actual results
 *
 ***/
print_abs(ab, wq)
struct abstract *ab;
struct waisresult *wq;
{

	char *tmp;

	if (!ab || !wq)
		return;

	if (!ab->url) 
		return;

	if (ab->title)
		printf("<A HREF=\"%s\">%s%s%s</A>%s\n",ab->url, STRONG,
			ab->title,ESTRONG, BREAK);
	else
		printf("<A HREF=\"%s\">%sNo Title Provided%s</A>%s\n",
			ab->url,STRONG,ESTRONG,BREAK);

	if (strlen(ab->data)>0)
		printf("%s%s",DEF,ab->data);

	printf("%s%s%s --- score:(%s) lines:(%s) bytes:(%s)%s%s",
		DEF,BOLD,ITAL,wq->score,wq->numofl, wq->numofb,EITAL,EBOLD);

	printf("%s\n",BREAK);
	printf("%s%s --- %s%s\n",DEF,ITAL,ab->url, EITAL); 

	tmp=(char *) calloc((strlen(wq->dir) + strlen(wq->fname) +1),sizeof(char));
	sprintf(tmp,"%s%s",wq->dir,wq->fname);

	do_remarks(tmp);

	free(tmp);

	printf("%s\n",PARA);
}

/***
 * strupr()
 *	Convert string to uppercase 
 *
 ***/

char *strupr(buf)
char *buf;
{
    int i;
    char *buf2;

    if (!buf)
	return(0);

	buf2=strdup(buf);

    i=0;
    while (*(buf2+i) != '\0') { 
	if ( (*(buf2+i) >= 'a') && (*(buf2+i) <= 'z'))
	   *(buf2+i) -= 32;
        i++;
    }	
    return(buf2);
}

/***
 * strlowu
 *	Convert string to lower case with first letter capitolised
 ***/
char *strlowu(buf)
char *buf;
{

	char *buf2;
	int i=0;

	buf2 = strlow(buf);
	while (!isalpha(*(buf2+i))) i++;
	if (islower(*(buf2+i)))
		*(buf2+i) -= 32;
	return(buf2);
}

/***
 * strlow()
 *	Convert string to lowercase 
 *
 ***/

char *strlow(buf)
char *buf;
{
    int i;
    char *buf2;

    if (!buf)
	return(0);

	buf2=strdup(buf);

    i=0;
    while (*(buf2+i) != '\0') { 
	if ( (*(buf2+i) >= 'A') && (*(buf2+i) <= 'Z'))
	   *(buf2+i) += 32;
        i++;
    }	
    return(buf2);
}

/***
 * suckstr()
 * 	Suck out the text string between the Title and Header HTML tags
 *	and return it.
 ***/

char *suckstr(fbuf,type, size, pos)
char *fbuf;
int *type;
long size;
unsigned long *pos;
{

	unsigned char headflag=0, titleflag=0;
	char *s, t[6];
	int i=0, tag=0;
	unsigned long l;

	l = *pos;

	if (!(s = (char *) calloc(LEN,sizeof(char)))) {
		printf("Error mallocing\n");
		return(NULL);
	}
	
	*type=0; i=0;

	while (l < size) {
		if (*(fbuf+l)=='<') { 
			tag=1; 
		        if (headflag && (strlen(s)>0)) { 
				*type=THEAD;
				headflag=0;
				*pos=l;
				return(s);
			  } else if (titleflag && (strlen(s)>0)) {
				titleflag=0;
				*type=TTITLE;
				*pos=l;
				return(s);
			  }
			  l++;
			  while (isspace(*(fbuf+l))) {
				l++;  
			  } 

		  	  switch (toupper(*(fbuf+l))) {
		     	      case '/' : while ((l<size) && (*(fbuf+l)!='>')) 
				     		l++;
				     	  l--;	
				     	  break;
		     		case 'H' : {
				    l++;
	    			    switch (*(fbuf+l)) {
					case '1': headflag=1;break;
					case '2' : headflag=1;break;
					case '3' : headflag=1;break;
				    }			 
			        }
			        break;
		   		case 'T' : {
					memset(t, 0, 6);
					strncpy(t, fbuf+l,5);
					if (strncmp(strupr(t),"TITLE",5)==0) {
						l+=4;	
						titleflag=1;
					}
							     
		              	}
		   	}
			  
		} else if (*(fbuf+l)=='>') {
			while (isspace(*(fbuf+(l+1)))) {
				l++;  
			} 
			tag=0; 
	        } else if (!tag && headflag && (i<LEN)) 
			*(s+i++) = *(fbuf+l);	
	        else if (!tag && titleflag && (i<LEN)) 
			*(s+i++) = *(fbuf+l);	
	        l++;
	}
	*pos=0;
	return(NULL);
}

/***
 * urlstr()
 *	Make url string from path name and WEBHOME
 ***/

char *urlstr(fname)
char *fname;
{

	char *tmp, *s;

	if (strncmp(fname,WEBTOP,strlen(WEBTOP))==0)
		tmp = fname+strlen(WEBTOP);
	else { 
		if (strncmp(fname,"./",2)==0)
			tmp=fname+1;
		else
			tmp=fname;
	}
			
	s = (char *) malloc(strlen(tmp)+strlen(TOPURL)+1);
	sprintf(s,"%s%s",TOPURL,tmp);

	return(s);
}

/***
 * printht()
 *	Print header and trailer from a file.
 ***/

void printht(fname)
char *fname;
{
	FILE *f1;
	char line[LEN];

	if ((f1 = fopen(fname,"r"))==NULL) {
		printf("Error opening file:%s<br>\n",fname);
		return;
	}
	
	while (fgets(line,LEN,f1)!=NULL)
		printf("%s",line);
	fclose(f1);

	return;
}

/***
 * do_remarks(fname)
 *
 *	fname - file name of hitfile
 *
 *	Use directory part of fname to get the path name of the remarks
 *	file and get the title. Print this title in HTML format.
 ***/

void do_remarks(fname)
char *fname;
{
	char *str, *remarks, *path, *p, *fbuf;
	int type;
        FILE *f1 , *f9;
	struct stat fstat;
	unsigned long pos=0;

	path = strdup(fname);
	p = strrchr(path,'/');
	if (p==NULL) {
		remarks=(char *) malloc(strlen(HWREMARKS)+2);
		sprintf(remarks,"/%s",HWREMARKS);
	} else {
	   *p=0;
	   remarks= (char *) malloc(strlen(path) + strlen(HWREMARKS)+2);
	   sprintf(remarks,"%s/%s",path,HWREMARKS);
	}

/* if no remarks file , do nothing */
	
	if (stat(remarks,&fstat)!=0) 
		return;

	if ((f1=fopen(remarks,"r"))==NULL)
		return;
	else {
		fbuf=(char *) malloc(fstat.st_size);
	        fread(fbuf, fstat.st_size,1,f1);
		fclose(f1);

		str = suckstr(fbuf,&type,fstat.st_size, &pos);
		free(fbuf);

		if (type==TTITLE && str!=NULL) 
			printf("\n%s%s---<A HREF=\"%s\">%s%s%s</A>",BREAK,DEF,urlstr(remarks),ITAL,str,EITAL);
		else 
			printf("\n%s%s---<A HREF=\"%s\">%sAdditional Information%s</A>",BREAK,DEF,urlstr(remarks),ITAL,EITAL);
		free(remarks);
		return;
	} 
}

/***
 * get_path(fname)
 *	returns a pointer to PathName of fname less the fname
 *
 ***/
char *get_path(fname)
char *fname;
{
	char *p, *path;

  	path = strdup(fname);
	p = strrchr(path, '/');
	if (p==NULL)
		path = "./";
	else
		*p=0;

	return(path);
}
/***
 * show_it(dir)
 *	If the file DONT_SHOW exists in this directory, then do not
 *	create an index for this file.
 *
 ***/
show_it(dir)
char *dir;
{
	char *path;
	int result;
	struct stat fstat;

	path = (char *) calloc(strlen(dir)+strlen(DONT_SHOW)+2, sizeof(char));	

	sprintf(path,"%s%s", dir, DONT_SHOW);
	
	result = stat(path, &fstat);
	free(path);
	return(result);	/* 0 = exists, -1 = No file */
}

/***
 * fix(str)
 *	
 *	str - string to scan for shell special chars.
 *
 *	Escape shell special chars
 *
 ***/
char *fix(str)
char *str;
{
	int i=0, j=0 , n;
	char *t;

	t = calloc(512, sizeof(char));

	while (*(str+i)) {
		switch (*(str+i)) { 
			case '-'  : *(t+j++) = '\\'; break;
                        case  '"' : *(t+j++) = '\\'; break;
                        case '\\' : *(t+j++) = '\\';
		}
		*(t+j++) = *(str+i++);
	}
	return(t);

}

